# 项目：standard library
# 模块：fuck gfw
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2017-02-05 19:16
# 修订：2017-02-22
'''
功夫网是档在我们和外部世界之间的一堵墙。幸好有好人提供了一把梯子，供我们翻墙
'''
from orange import *
import os

repo='git@github.com:racaljk/hosts.git'

def main():
    if os.name=='posix':
        src=Path('~/.fkgfw')
        dest=Path('/private/etc/hosts')
    else:
        src=Path('%appdata%/fkgfw')
        dest=Path('%SystemRoot%/System32/drivers/etc/hosts')
    src.ensure()
    src.chdir()
    if not (src / 'hosts').exists():
        os.system('git clone %s'%(repo))
    else:
        (src/'hosts').chdir()
        os.system('git pull')
        
    if os.name=='posix':
        os.system('sudo cp %s %s'%(src/'hosts/hosts',dest))
    else:
        dest.text=(src/'hosts/hosts').text
        os.system('ipconfig /flushdns')
    print(*dest.lines[:10],sep='\n')
    
if __name__=='__main__':
    main()
    

    



# 项目：标准库函数
# 模块：macos的plist生成器
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2016-09-06 23:27
# 修改：2016-9-17 使用plistlib 来写入plist文件

import sys
from plistlib import *
from orange import *
import sys

def proc(filename,label,*args):
    filename=str(Path(filename).with_suffix('.plist'))
    with open(filename,'wb') as fn:
        dump(Dict(Label=label,
                  KeepAlive=True,
                  ProgramArguments=args),fn)

def main():
    args=sys.argv
    if len(args)<4:
        print('Usage:\n'
        '\t%s plist-file-name label program args'%(args[0]))
    else:
        proc(*args[1:])
    

if __name__=='__main__':
    main()



# 项目：standard python lib
# 模块：path and file
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2016-03-11 12:21
# 修改：2016-04-13 21:01
# 修改：2016-08-13 新增__iter__ 和 extractall功能
# 修订：2016-11-19 修改Path的实现方式 

import pathlib
import os
from codecs import BOM_UTF8,BOM_LE,BOM_BE
from .click import *

BOM_CODE={
    BOM_UTF8:'utf_8',
    BOM_LE:'utf_16_le',
    BOM_BE:'utf_16_be',
    }
    
DEFAULT_CODES='utf8','gbk','utf16','big5'

def is_installed(file_name):
    '''
    确认指定的文件是否已被安装。
    '''
    from sysconfig import get_path
    paths=[get_path(name) for name in ('platlib','scripts')]
    if os.name=='nt':
        file_name=file_name.lower()
        paths=[path.lower() for path in paths]
    return any([file_name.startswith(path) for path in paths])

def is_dev(cmd=None):
    import sys
    cmd=cmd or sys.argv[0]
    if('wsgi' in cmd):
        return False
    return 'test' in cmd or (not is_installed(cmd))

def decode(d):
    '''
    对指定的二进制，进行智能解码，适配适当的编码。按行返回字符串。
    '''
    for k in BOM_CODE:
        if k==d[:len(k)]:
            return d[len(k):].decode(BOM_CODE[k])
    for encoding in DEFAULT_CODES:
        try:
            return d.decode(encoding)
        except:
            pass
    raise Exception('解码失败')

_Parent= pathlib.WindowsPath if os.name=='nt' else pathlib.PosixPath

class Path(_Parent):
    __slots__=()
    def __new__(cls,path='.',*args,**kwargs):
        if isinstance(path,str):
            if path.startswith('~'):  # 支持用户目录开头
                path=os.path.expanduser(path)
            elif path[0] in('%','$'): # 支持环境变量转义
                path=os.path.expandvars(path)
        return super().__new__(cls,path,*args,**kwargs)
    
    def read(self,*args,**kwargs):
        '''以指定的参数读取文件'''
        with self.open(*args,**kwargs)as fn:
            return fn.read()

    def ensure(self,parents=True):
        '''确保目录存在，如果目录不存在则直接创建'''
        if not self.exists():
            self.mkdir(parents=parents)
            
    @property
    def text(self):
        '''读取文件，并返回字符串'''
        return decode(self.read('rb'))

    @text.setter
    def text(self,text):
        '''写入文本文件'''
        self.write(text=text)
        
    @property
    def lines(self):
        '''按行读取文件'''
        return self.text.splitlines()

    @lines.setter
    def lines(self,lines):
        '''按行写入文件'''
        self.write(*lines)
        
    def write(self,*lines,text=None,data=None,encoding='utf8',
              parents=False):
        '''写文件'''
        if parents:
            self.parent.ensure()
        if lines:
            text="\n".join(lines)
        if text:
            data=text.encode(encoding)
        if data:
            with self.open('wb')as fn:
                fn.write(data)

    def sheets(self,index=None):
        ''' 提供读取指定worksheet的功能，其中index可以为序号，
            也可以为表的名称。'''
        import xlrd
        book=xlrd.open_workbook(filename=str(self))
        if isinstance(index,int):
            sheet=book.sheet_by_index(index)
        elif isinstance(index,str):
            sheet=book.sheet_by_name(index)
        return sheet and sheet._cell_values
        
    def iter_sheets(self):
        '''如果指定的文件为excel文件，则可以迭代读取本文件的数据。
        返回：表的索引、表名、数据'''
        import xlrd 
        book=xlrd.open_workbook(filename=str(self))
        for index,sheet in enumerate(book.sheets()):
            yield index,sheet.name,sheet._cell_values

    @property
    def xmlroot(self):
        '''如果指定的文件为xml文件，则返回本文件的根元素'''
        import lxml.etree
        return lxml.etree.parse(str(self)).getroot()
        
    def __iter__(self):
        '''根据文件的不同，迭代返回不同的内容。支持如下文件：
        1、文本文件，按行返回文本
        2、Excel文件，返回表索引、表名及表中数据
        3、目录，则返回本目录下所有文件
        4、del文件，按行返回数据。
        5、csv文件，按行返回数据。
        '''
        if self.is_dir():
            return self.glob('*')
        suffix=self.lsuffix
        if suffix.startswith('.xls'):
            return self.iter_sheets()
        elif suffix=='.xml':
            return self.xmlroot.iterchildren()
        elif suffix=='.del':
            import re
            none_pattern=re.compile(",(?=,)")
            for line in self.lines:
                if line:
                    yield eval(none_pattern.sub(",None",line))
        elif suffix=='.csv':
            import csv
            with self.open() as fn:
                yield from csv.reader(fn)

    def extractall(self,path='.',members=None):
        '''如本文件为tar打包文件，则解压缩至指定目录'''
        import tarfile
        path=str(Path(path))
        tarfile.open(str(self),'r').extractall(path,members)
        
    @property
    def lsuffix(self):
        '''返回小写的扩展名'''
        return self.suffix.lower()
        
    @property
    def pname(self):
        '''返回不带扩展名的文件名'''
        return self.with_suffix("").name
    
    def rmtree(self):
        '''删除整个目录'''
        import shutil
        shutil.rmtree(str(self))

    def chdir(self):
        if self.is_dir():
            os.chdir(str(self))

    def write_xlsx(self):
        from orange.xlsx import Book
        return Book(str(self))

@command(description='Windows 格式文件转换为 Unix 文件格式')
@arg('files',nargs='*',help='待转换的文件',metavar='file')
def convert(files):
    for file in files:
        Path(file).lines=Path(file).lines
        print('转换文件"%s"成功'%(file))




# 项目：Excel写入模块封装
# 模块：xlsxwriter的封装
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2016-10-12 08:24

from orange import *
from xlsxwriter import *
from xlsxwriter.format import Format
from xlsxwriter.worksheet import *

Pattern=R/r'([A-Z]{1,2})(\d*)([:_]([A-Z]{1,2})(\d*))?'
Row=R/r'(\{([+-]?\d+)\})'

DefaultFormat=(('currency',{'num_format':'#,##0.00'}),
                ('rate',{'num_format':'0.0000%'}),
                ('title',{'font_name':'黑体','font_size':16,
                          'align':'center'}),
                ('h2',{'font_name':'黑体','font_size':12,
                          'align':'center'}),
                ('mh2',{'font_name':'黑体','font_size':12,'text_wrap':True,
                          'align':'center','valign':'vcenter'}),
                ('percent',{'num_format':'0.00%'}),
                ('date',{'num_format':'yyyy-mm-dd'}),
                ('time',{'num_format':'hh:mm:ss'}),
                ('number',{'num_format':'#,##0'}),
                ('datetime',{'num_format':'yyyy-mm-dd hh:mm:ss'}),
                ('timestamp',{'num_format':'yyyy-mm-dd hh:mm:ss.0'}))

class Book(Workbook):
    ''' 对Xlsxwriter模块进一步进行封装'''
    def __init__(self,filename=None,**kw):
        if isinstance(filename,str):
            filename=str(Path(filename))
        super().__init__(filename,**kw)
        self._worksheet=None    # 设置当前的工作表为空
        self._worksheets={}     # 设置当前的工作表清单为空
        self._formats={}
        for name,val in DefaultFormat:
            self.add_format(val,name)

    def add_format(self,properties,name=None):
        _format=super().add_format(properties)
        if name:
            self._formats[name]=_format
            _format.name=name
            _format.properties=properties
        return _format

    def add_formats(self,properties):
        for name,property in properties.items():
            self.add_format(property,name)

    def set_widths(self,widths):
        [self.set_columns(col,width=width)for col,width in widths.items()]
        
    def set_columns(self,*columns,width=None,cell_format=None,options=None):
        ''' 设置当前工作表的列属性，允许同时设置多个，使用方法如下：
        book.set_columns('A:C','E:D','G:H',width=12)
        '''
        options=options or {}
        for column in columns:
            self.worksheet.set_column(column,width,cell_format,options)

    def newline(self):
        '''换行'''
        return self+1
        
    @property
    def worksheet(self):
        '''当前工作表'''
        return self._worksheet

    @worksheet.setter
    def worksheet(self,name):
        '''切换当前工作表'''
        worksheet=self._worksheets.get(name,None)
        if not worksheet:
            worksheet=self.add_worksheet(name)
            worksheet.row=1
            self._worksheets[name]=worksheet
        self._worksheet=worksheet

    @convert_range_args
    def write(self,first_row,first_col,last_row=None,last_col=None,
               value=None,cell_format=None):
        if isinstance(cell_format,str):
            cell_format=self._formats.get(cell_format)
        if(last_row is None)or(first_row==last_row and \
                               first_col==last_col):
            if isinstance(value,(tuple,list)):
                self.worksheet.write_row(first_row,first_col,
                                         value,cell_format)
            else:
                if isinstance(value,str) and value.startswith('='):
                    value=Row/value%self._convert   # 使用正则表达式替换
                self.worksheet.write(first_row,first_col,value,
                                     cell_format)
        else:
            self.worksheet.merge_range(first_row,first_col,last_row,
                                       last_col,value,cell_format)
    
    def _write(self,range,value,format=None):
        '''写入单元格，这个函数将被放弃，由write替代'''
        if isinstance(format,str):  # 格式代码检查
            format=self._formats.get(format)
        if ':' in range and not isinstance(value,(dict,tuple,list)):
            self.worksheet.merge_range(range,value,format)
        if ':' not in range:
            if isinstance(value,(list,tuple)):
                self.worksheet.write_row(range,value,format)
            else:
                if isinstance(value,str) and value.startswith('='):
                    value=Row/value%self._convert   # 使用正则表达式替换
                self.worksheet.write(range,value,format)

    def _convert(self,match):
        return str(int(match.groups()[1])+self.row)

    def __add__(self,val):
        '''向前移动当前行'''
        self.worksheet.row+=val
        return self
    
    def __sub__(self,val):
        '''向后移动当前行'''
        self.worksheet.row-=val
        return self

    @property
    def row(self):
        '''获取当前工作表的行'''
        return self.worksheet.row

    @row.setter
    def row(self,currow):
        '''设置当前工作表的行'''
        self.worksheet.row=currow
    
    def __setitem__(self,name,val):
        '''向指定行写入数据'''
        if not isinstance(val,tuple):
            val=(val,)
        if isinstance(name,str):
            match=Pattern.fullmatch(name)
            if match:
                r=list(match.groups())
                if not r[1]:
                    r[1]=str(self.row)
                rg=''.join(r[:2])
                if r[3]:
                    if not r[4]:
                        r[4]=str(self.row)
                    rg='%s:%s%s'%(rg,r[3],r[4])
                self.write(rg,*val)
            else:
                raise Exception('单元格格式不正确')
        elif isinstance(name,int):
            self.write(self.row-1,name,None,None,*val)
        elif isinstance(name,tuple):
            args=list(name)
            args.extend([None]*4)
            self.write(*args[:4],*val)

    def __setattr__(self,name,val):
        '''向指定行写入数据'''
        if Pattern==name:
            self[name]=val
        else:
            super().__setattr__(name,val)

    def iter_rows(self,*datas,step=1):
        '''按行写入数据'''
        for d in zip(*datas):
            yield d
            self+=step

    @property
    def table(self):
        return self.worksheet.table

    @convert_range_args
    def set_border(self, first_row, first_col, last_row, last_col,
                  left=None,right=None,bottom=None,top=None,border=2,
                  inner=1):
        self.worksheet._check_dimensions(first_row,first_col)
        self.worksheet._check_dimensions(last_row,last_col)
        if border:
            left=left or border
            right=right or border
            bottom=bottom or border
            top=top or border
        table=self.table        
        default_format={"left":inner,"right":inner,"top":inner,
                               "bottom":inner}
        def _replace(r,c,**kw):
            row=table[r]
            cell=row.get(c,cell_blank_tuple(None))
            fmt=cell.format
            kw['top']=top if r==first_row else inner
            kw['bottom']=bottom if r==last_row else inner
            kw['left']=left if c==first_col else inner
            kw['right']=right if c==last_col else inner
            name=''.join([str(kw[name]) for name in \
                          'left top right bottom'.split()])
            if fmt and hasattr(fmt,'name'):
                name=name+'-'+fmt.name
            if not name in self._formats:
                if fmt and hasattr(fmt,'properties'):
                    a=fmt.properties.copy()
                    a.update(kw)
                else:
                    a=kw
                new_fmt=self.add_format(a,name)
            else:
                new_fmt=self._formats.get(name)
            row[c]=cell._replace(format=new_fmt)
        [_replace(r,c)for r in range(first_row,last_row+1)\
         for c in range(first_col,last_col+1)]
         
    @convert_range_args
    def add_table(self,first_row,first_col,last_row,last_col,\
                  sheet=None,**kwargs):
        '''添加图表，如sheet为空，则使用默认的工作表'''
        if sheet:
            self.worksheet=sheet
        columns=kwargs.get('columns')
        if columns:
            new_columns=[]
            for idx,column in enumerate(columns):
                if 'width' in column:
                    self.set_columns("{0}:{0}".format(
                        xl_col_to_name(idx+first_col)),\
                        width=column.get('width'))
                format=column.get("format")
                if format and isinstance(format,str):
                    new_column=column.copy()
                    new_column['format']=self._formats.get(format)
                    new_columns.append(new_column)
                else:
                    new_columns.append(column)
            kwargs['columns']=new_columns
            last_col=first_col+len(columns)-1
        if 'data' in kwargs:
            last_row=first_row+len(kwargs['data'])
            if kwargs.get('total_row',False):
                last_row+=1
        self.worksheet.add_table(first_row,first_col,last_row,last_col,kwargs)
        
if __name__=='__main__':
    with Book('test.xlsx') as book:
        book.worksheet='test1'
        columns=[{'header':'title','format':'currency','width':12},
                     {'header':'name','format':'percent'}]
        data=[('a','b'),(1,2)]
        book.add_table('A1',data=data,columns=columns)



# Xlsx报表库
# 主要是对xlsxwriter进行进一步封装
# 作者：黄涛
# 创建：2015-12-04

from xlsxwriter.utility import xl_col_to_name
from xlsxwriter import Workbook
from xlsxwriter.worksheet import convert_range_args,Worksheet,\
     convert_cell_args,convert_column_args
from orange import *

# 预定义格式
DefaultFormats=(('currency',{'num_format':'#,##0.00'}),
                ('rate',{'num_format':'0.0000%'}),
                ('title',{'font_name':'黑体','font_size':16,
                          'align':'center'}),
                ('h2',{'font_name':'黑体','font_size':12,
                          'align':'center'}),
                ('mh2',{'font_name':'黑体','font_size':12,
                          'align':'center','valign':'vcenter'}),
                ('percent',{'num_format':'0.00%'}),
                ('date',{'num_format':'yyyy-mm-dd'}),
                ('time',{'num_format':'hh:mm:ss'}),
                ('number',{'num_format':'#,##0'}),
                ('datetime',{'num_format':'yyyy-mm-dd hh:mm:ss'}),
                ('timestamp',{'num_format':'yyyy-mm-dd hh:mm:ss.0'}))

class XlsxReport():
    '''
    导出Excel报表，主要的用法：
    data=(('data11','data12'),
          ('data21','data22'))
    columns=[{'header':'标题1',
              'width':11,},
             {'header':'标题2',
              'width':13,
              'format':'currency'}]
    with XlsxReport('xlsx_name')as rpt:
        rpt.add_table('A1','sheetname',columns=columns,data=data)
    '''
    def __init__(self,*args,**kwargs):
        deprecation("XlsxReport","Book")
        self.book=Workbook(*args,**kwargs)
        self.formats={}
        self.add_formats(DefaultFormats)
        self.sheet=None
        self.sheets={}

    def __enter__(self):
        return self

    def __exit__(self,_type,value,trace):
        self.close()

    def ensure_sheet(self,sheet):
        self.get_sheet(sheet)
        if not sheet in self.sheets:
            self.sheets[sheet]=Sheet(self)
        return self.sheets.get(sheet)

    def get_sheet(self,sheetname):
        '''获取工作表，并设为默认工作表'''
        if isinstance(sheetname,Worksheet):
            return sheetname
        for sheet in self.book.worksheets():
            if sheet.name==sheetname:break
        else:
            sheet=self.book.add_worksheet(sheetname)
        self.sheet=sheet
        return sheet

    def set_row(self,row,height=None,format=None,options={}):
        if format:
            format=self.formats.get(format,format)
        self.sheet.set_row(row,height,format,options)

    @convert_column_args
    def set_column(self,firstcol,lastcol,width=None,
                   format=None,options={}):
        if format:
            format=self.formats.get(format,format)
        self.sheet.set_column(firstcol,lastcol,width,format,options)
    
    @convert_range_args
    def mwrite(self,first_row,first_col,last_row,last_col,\
               value,format=None):
        '''合并写入到默认的工作表中'''
        if format:
            format=self.formats.get(format,format)
        self.sheet.merge_range(first_row,first_col,last_row,last_col,
                               value,format)

    @convert_range_args
    def write_formulas(self,*args,**kwargs):
        pass
    
    @convert_cell_args
    def write_formula(self,row,col,formula,format=None,value=0):
        if format:
            format=self.formats.get(format,format)
        self.sheet.write_formula(row,col,formula,format,value)
        
    @convert_cell_args
    def write(self,row,col,value,format=None):
        '''单一单元格写入'''
        if format:
            format=self.formats.get(format,format)
        self.sheet.write(row,col,value,format)

    @convert_cell_args
    def rwrite(self,row,col,values,format=None):
        '''按行写入'''
        if format:
            format=self.formats.get(format,format)
        self.sheet.write_row(row,col,values,format)
        
    def close(self):
        '''关闭文件'''
        self.book.close()

    def add_formats(self,args):
        '''添加格式'''
        if isinstance(args,dict):
            args=args.items()
        self.formats.update({name:self.book.add_format(format) for \
                    name,format in args})
                      
    @convert_range_args
    def add_table(self,first_row,first_col,last_row,last_col,\
                  sheet=None,**kwargs):
        '''添加图表，如sheet为空，则使用默认的工作表'''
        sheet=self.get_sheet(sheet)if sheet else self.sheet
        columns=kwargs.get('columns')
        if columns:
            new_columns=[]
            for idx,column in enumerate(columns):
                if 'width' in column:
                    sheet.set_column("{0}:{0}".format(
                        xl_col_to_name(idx+first_col)),\
                        column.get('width'))
                format=column.get("format")
                if format and isinstance(format,str):
                    new_column=column.copy()
                    new_column['format']=self.formats.get(format)
                    new_columns.append(new_column)
                else:
                    new_columns.append(column)
            kwargs['columns']=new_columns
            last_col=first_col+len(columns)-1
        if 'data' in kwargs:
            last_row=first_row+len(kwargs['data'])
            if kwargs.get('total_row',False):
                last_row+=1
        sheet.add_table(first_row,first_col,last_row,last_col,kwargs)

Pattern=R/r'([A-Z]{1,2})(\d*)([:_]([A-Z]{1,2})(\d*))?'

class Sheet:
    def __init__(self,rpt):
        self.rpt=rpt
        self.row=1

    def write(self,range,value,format=None):
        if ':' in range and not isinstance(value,(dict,tuple,list)):
            self.rpt.mwrite(range,value,format)
        if not ':' in range:
            if isinstance(value,(list,tuple)):
                self.rpt.rwrite(range,value)
            else:
                if isinstance(value,str) and value.startswith('='):
                    value=value.format(self.row)
                self.rpt.write(range,value,format)

    def __add__(self,val):
        self.row+=val
        return self
    
    def __sub__(self,val):
        self.row-=val
        return self
    
    def __setitem__(self,name,val):
        match=Pattern.fullmatch(name)
        if match:
            r=list(match.groups())
            if not r[1]:
                r[1]=str(self.row)
            rg=''.join(r[:2])
            if r[3]:
                if not r[4]:
                    r[4]=str(self.row)
                rg='%s:%s%s'%(rg,r[3],r[4])
            if not isinstance(val,tuple):
                val=(val,)
            self.write(rg,*val)
        else:
            raise Exception('单元格格式不正确')

    def __setattr__(self,name,val):
        try:
            self[name]=val
        except:
            super().__setattr__(name,val)

    def iter_rows(self,count):
        for i in range(count):
            yield self
            self+1



# 项目：标准库函数
# 模块：安装模块
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2016-03-12 18:05

import distutils.core
import os
import setuptools
import pip
from orange import Path,Ver

def get_path(pkg,user=True):
    ''' 返回指定包的参数配置目录和数据目录'''
    if os.name=='posix':
        if user:
            root=Path('~')
            return root,root/ ('.%s'%(pkg))
        else:
            root=Path('/usr/local')
            return root/'etc',root/'var'/pkg
    else:
        if user:
            root=Path('~/AppData')
            return root /'Roaming' ,root / 'Local' / pkg
        else:
            root=Path('%programdata%') / pkg
            return root,root
            
def run_pip(*args):
    pip.main(list(args))
    
def run_setup(*args):
    distutils.core.run_setup('setup.py',args)

DEFAULT={'author':'huangtao',
         'author_email':'huangtao.sh@icloud.com',
         'platforms':'any',
         'license':'GPL',}

def _get_requires():
    result=[]
    for fn in Path('.').glob('*/requires.txt'):
        for row in fn.lines:
            i=row.find('#')
            if i>-1:
                row=row[:i]
            row=row.strip()
            if row:
                result.append(row)
    return result
            
def setup(version=None,packages=None,after_install=None,
          scripts=None,install_requires=None,
          cscripts=None,gscripts=None,
          **kwargs):
    if cscripts or gscripts:
        entry_points=kwargs.get('entry_points',{})
        if cscripts:
            entry_points['console_scripts']=cscripts
        if gscripts:
            entry_points['gui_scripts']=gscripts
        kwargs['entry_points']=entry_points
    for k,v in DEFAULT.items():
        kwargs.setdefault(k,v)
    if not packages:
        # 自动搜索包
        packages=setuptools.find_packages(exclude=('testing',
                                                   'scripts'))
    if not version:
        # 自动获取版本
        version=str(Ver.read_file())
    if not install_requires: # 从repuires.txt 中获取依赖包
        install_requires=_get_requires()
    if not scripts:
        scripts=[str(path) for path in Path('.').glob('scripts/*')]
    # 安装程序 
    dist=distutils.core.setup(scripts=scripts,packages=packages,
            install_requires=install_requires,
            version=version,**kwargs)
    # 处理脚本

    if 'install' in dist.have_run and os.name=='posix' \
      and scripts:
        from sysconfig import get_path
        prefix=Path(get_path('scripts'))
        for script in scripts:
            script_name=prefix/(Path(script).name)
            if script_name.lsuffix in ['.py','.pyw']\
              and script_name.exists():
                script_name.replace(script_name.with_suffix(''))
    if 'install' in dist.have_run and after_install:
        after_install(dist)



from orange import *
import os
import sys

LINKS={'bin':'bin',
       'emacsd/emacs':'.emacs',
       'conf/gitconfig':'.gitconfig',
       'conf/ssh':'.ssh',
       'conf/pypirc':'.pypirc',
       'conf/pip':'.pip',
       }
    
WIN32_LINKS={'conf/pip':'AppData/Roaming/pip',
           'conf/vimrc_win':'_vimrc',
           }
    
DARWIN_LINKS={'conf/vimrc_mac':'.vimrc',}

def win_init():
    # 修改注册表，增加.PY 和.PYW 为可执行文件
    from orange.regkey import HKLM,REG_SZ,HKCU
    with HKCU/'GNU/Emacs' as key:
        home=str(Path('~'))
        key['HOME']=home,REG_SZ
        print('设置 Emacs 的 HOME 目录完成。')

    with HKLM/r'SYSTEM\CurrentControlSet\Control\Session Manager\Environment' as key:
        pathext=key['PATHEXT'][0]
        for ext in ('.PY','.PYW'):
            if ext not in set(pathext.split(';')):
                pathext+=';'+ext
        key['PATHEXT']=pathext,REG_SZ
        
        
def do_link():
    if sys.platform=='win32':
        LINKS.update(WIN32_LINKS)
    elif sys.platform=='darwin':
        LINKS.update(DARWIN_LINKS)

    home=Path(os.path.expanduser('~'))
    src=home / 'OneDrive'

    for source,dest in LINKS.items():
        s= src / source
        d= home /dest
        if not d.exists()and s.exists():
            d.symlink_to(s,s.is_dir())
            print('创建连接文件：%s -> %s'%(d,s))
def main():
    do_link()
    if sys.platform=='win32':
        win_init()
    elif sys.platform=='darwin':
        darwin_init()
        
if __name__=='__main__':
    main()




# 项目：标准库函数
# 模块：调试模块
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2016-03-12 17:25

from functools import wraps
import logging
import sys
from .path import is_dev

__all__='decorator','trace','config_log','ensure','info','fprint','verbose'

info=logging.info

def decorator(decorator):
    def _decorator(func):
        @wraps(func)
        def __decorator(*args,**kwargs):
            return decorator(func,*args,**kwargs)
        return __decorator
    return _decorator

@decorator
def trace(func,*args,**kwargs):
    print('The function %s is called'%(func.__name__))
    print('With arguments %s %s'%(args,kwargs))
    result=func(*args,**kwargs)
    print('The result is %s'%(result))
    return result

def verbose():
    config_log(format='%(message)s',level=20)
    
def config_log(**kwargs):
    kwargs.setdefault('datefmt','%Y-%m-%d %H:%M:%S')
    kwargs.setdefault('format','%(levelname)-9s %(message)s')
    kwargs.setdefault('level',10 if is_dev() else 30)
    logging.basicConfig(**kwargs)

def ensure(cond,msg,level="error"):
    if not cond:
        getattr(logging,level)(msg)
        if level in ('error','critical','fatal'):
            raise Exception(msg)

def fprint(*args,sep=' ',end='\n',replace='',**kw):
    '''force print, used in Windows,解决打印GBK问题
    其中replace是遇无法打印字符的替代字符'''
    try:
        print(*args,sep=sep,end=end,**kw)
    except UnicodeEncodeError:
        s = sep.join(str(x) for x in args)
        for c in s:
            try:
                print(c,end='')
            except UnicodeEncodeError:
                print(replace, end='')
        print(end,end='')



# 项目：标准库函数
# 模块：网而数据分析
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2016-11-19 16:56


from urllib import parse,request
from bs4 import BeautifulSoup as BeautifulSoup
from orange.path import *
from threading import Thread

def url_open(url,data=None,features='lxml',proc=None,**kw):
    '''打开网页，并分析其内容
    url: 指定的网页
    data: 查询参数
    proc: 分析程序
    kw:   其他传递给BeautifulSoup的参数
    '''
    if data:
        url='%s?%s'%(url,parse.urlencode(data))
    with request.urlopen(url) as fn:
        data=fn.read()
        markup=decode(data)
    soup=BeautifulSoup(markup,features,**kw)
    if callable(proc):
        proc(soup)
    return soup

def turl_open(*args,**Kw):
    '''以线程的方式打开网页，函数说明同 url_open'''
    Thread(target=url_open,args=args,kwargs=kw).start()
    



# 项目：Pytohon工具包
# 模块：安装模块
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2015-09-03 16:02
# 修改：2016-03-08 16:47
# 修改：2016-04-13 21:07
# 修改：2016-9-7 将通过shell调用pip命令调整为直接引入pip模块

import os
import sys
import re
from orange import *
from orange.parseargs import *
from orange.deploy import *

def pyclean():
    for path in ('build','dist','*egg-info'):
        for p in Path('.').glob(path):
            p.rmtree()
            print('Path %s have been deleted!'%(p))

RootPath='~/OneDrive/pylib'
Pattern=re.compile(r'\d+(\.\d+)*([ab]\d+)?')

def find_ver(path):
    v=Pattern.search(path.name)
    if v:
        return Ver(v.group())

def py_setup(packages,path,download,upgrade):
    root=Path(path)
    if download:
        run_pip('download','-d',str(root),*packages)
        #exec_cmd('pip','download -d %s %s'%(Path(path),
        #                      " ".join(packages)))
    elif upgrade:
        pip='pip' if os.name=='nt' else 'pip3'
        pkglist=read_shell('%s list -o'%(pip))
        for line in pkglist:
            pkg=line.split()
            if pkg:
                run_pip('install','-U',pkg[0])
    else:
        if packages:
            pkgs=[]
            for pkg in packages:
                pkg_path,pkg_ver=None,Ver('0.0')
                for file in root.glob('%s-*'%(pkg)):
                    info('Process file %s'%(file.name))
                    ver=find_ver(file)
                    info('Get ver %s'%(ver))
                    if ver>pkg_ver:
                        pkg_path=file
                        pkg_ver=ver
                if pkg_path:
                    if Path(pkg_path).lsuffix in ('.zip','.whl','.gz','.tar'):
                        pkgs.append(str(pkg_path))
                        info('Add file %s'%(pkg_path.name))
                    else:
                        print('%s 不是正常的包文件'%(pkg_path.name))
                else:
                    pkgs.append(pkg)
                    info('Add pkg %s'%(pkg))
            os.chdir('%s'%(root))
            run_pip('install',*pkgs)
            # exec_cmd('pip','install %s'%(" ".join(pkgs)))
        else:
            if Path('setup.py').exists():
                run_setup('install')
                pyclean()
            else:
                print('Can''t find the file setup.py!')

pysetup=Parser(
    Argument(
        'packages',
        help='python package',
        nargs='*',
        metavar='package'),
    Argument('-p','--path',
             default=RootPath,
             help='指定的目录'),
    Argument('-d','--download',
             help='默认的包目录',
             action='store_true'),
    Argument('-u','--upgrade',
             help='升级系统中已安装的软件包',
             action='store_true'),
    proc=py_setup,
    allow_empty=True)

if __name__=="__main__":
    pysetup()



# 项目：标准库函数
# 模块：命令行处理模块
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2017-01-18 21:34
# 修改：2017-02-05 为__call__函数增加位置参数，可以支持主程序为classmethod
# 修改：2017-02-11 调整相关函数名
 
from functools import partial

__all__='command','arg'

class _Command(object):
    kwargs={}            # ArgumentParser 的参数，由command函数生成
    allow_empty=False # 是否允许参数为空，如允许则argv为None的情况下也调用执行函数
    def __init__(self,run):
        self.run=run        # 可执行函数，命令行解析成功时调用  
        self.args=[]        # 参数列表
        self.subcommands=[] # 子命令列表
        
    @classmethod
    def wrapper(cls,cmd,*args,**kw):
        def _(fn):
            if not isinstance(fn,cls):
                fn=cls(fn)
            getattr(fn,cmd)(*args,**kw)
            return fn
        return _

    def add_subcommand(self,subcommand):
        self.subcommands.append(subcommand)
        
    def add_arg(self,*args,**kw):
        self.args.append((args,kw))

    def command(self,*subcommands,allow_empty=False,**kwargs):
        self.kwargs=kwargs                  # ArgumentParser 的参数
        self.subcommands=list(subcommands)  # 子命令
        self.allow_empty=allow_empty        # 是否允许命令行为空

    def proc_args(self,parser):
        for args,kw in reversed(self.args):
            parser.add_argument(*args,**kw)

    def __call__(self,*args,argv=None):
        import sys
        from argparse import ArgumentParser
        argv=argv or sys.argv[1:]
        parser=ArgumentParser(**self.kwargs)
        self.proc_args(parser)
        parser.set_defaults(proc=self.run)
        if self.subcommands:
            subparsers=parser.add_subparsers(help='Sub command')
            for subcommand in self.subcommands:
                help=subcommand._kw.pop('description')
                if help:
                    subcommand._kw['help']=help
                sub=subparsers.add_parser(subcommand.run.__name__,
                            **subcommand.kwargs)
                subcommand.proc_args(sub)
                sub.set_defaults(proc=subcommand.run)
        if self.allow_empty or argv:
            kwargs=dict(parser.parse_args(argv)._get_kwargs())
            proc=kwargs.pop('proc',None)
            if proc:
                try:
                    proc(*args,**kwargs)
                except Exception as e:
                    print(str(e))
        else:
            parser.print_usage()

command=partial(_Command.wrapper,'command')
arg=partial(_Command.wrapper,'add_arg')




# 项目：标准库函数
# 模块：时间模块
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2015-09-17 13:24
# 修改：2016-03-12 18:53
# 修改：2016-09-06 增加 ONEDAY,ONESECOND
# 修改：2016-11-19 将datetime 修改为按类实现

import datetime as dt
import time as _time
from .regex import R

__all__='UTC','LOCAL','now','datetime','FixedOffset','ONEDAY',\
        'ONESECOND','date_add','LTZ'

ZERO = dt.timedelta(0)      
ONEDAY = dt.timedelta(days=1)
ONESECOND = dt.timedelta(seconds=1)

# A class building tzinfo objects for fixed-offset time zones.
# Note that FixedOffset(0, "UTC") is a different way to build a
# UTC tzinfo object.

class FixedOffset(dt.tzinfo):
    """Fixed offset in minutes east from UTC."""

    def __init__(self, offset, name):
        self.__offset = dt.timedelta(minutes = offset)
        self.__name = name

    def utcoffset(self, dt):
        return self.__offset

    def tzname(self, dt):
        return self.__name

    def dst(self, dt):
        return ZERO

    def __repr__(self):
        timezone=self.__offset.total_seconds()//60
        return "UTC%+i:%02i"%(divmod(timezone,60)) if timezone else "UTC"
        

UTC=FixedOffset(0,'UTC')

# A class capturing the platform's idea of local time.

STDOFFSET = dt.timedelta(seconds = -_time.timezone)
if _time.daylight:
    DSTOFFSET = dt.timedelta(seconds = -_time.altzone)
else:
    DSTOFFSET = STDOFFSET

DSTDIFF = DSTOFFSET - STDOFFSET

class LocalTimezone(dt.tzinfo):

    def utcoffset(self, dt):
        if self._isdst(dt):
            return DSTOFFSET
        else:
            return STDOFFSET

    def dst(self, dt):
        if self._isdst(dt):
            return DSTDIFF
        else:
            return ZERO

    def tzname(self, dt):
        return _time.tzname[self._isdst(dt)]

    def _isdst(self, dt):
        tt = (dt.year, dt.month, dt.day,
              dt.hour, dt.minute, dt.second,
              dt.weekday(), 0, 0)
        stamp = _time.mktime(tt)
        tt = _time.localtime(stamp)
        return tt.tm_isdst > 0
    
    def __repr__(self):
        offset=STDOFFSET.total_seconds()//60
        return "UTC%+i:%02i"%(divmod(offset,60))

LTZ = LocalTimezone()
LOCAL =LTZ

class datetime(dt.datetime):
    '''日期类，支持从字符串转换'''
    def __new__(cls,year,month=0,day=0,hour=0,minute=0,second=0,
                microsecond=0,tzinfo=None):
        tzinfo=tzinfo or LTZ
        if all([year,month,day]):
            return super().__new__(cls,year,month,day,hour,minute,
                                   second,microsecond,tzinfo)
        else:
            if isinstance(year,datetime):
                return year
            elif isinstance(year,(dt.datetime,dt.time)):
                '''
                如果是DATETIME或TIME类型，检查是否有TZINFO信息，
                如无，则设为LTZ，否则直接返回
                '''
                args=list(year.timetuple()[:6])
                if year.tzinfo:
                    tzinfo=year.tzinfo
                args.extend([year.microsecond,tzinfo])
            elif isinstance(year,str):
                '''将字符串转换为DATETIME类型'''
                args=[int(x) for x in R/'\d+'/year]
            elif isinstance(year,(int,float)):
                '''将整数或浮点数转换成日期类型
                如果小于100000，则按EXCEL的格式转换；
                否则按UNIX TIMESTAMP 来转换'''
                if year<100000:
                    from xlrd.xldate import xldate_as_datetime
                    dd=cls(xldate_as_datetime(year,None))
                else:
                    dd=cls.fromtimestamp(year)
                return dd
            if len(args)==8:
                tzinfo=args.pop(7)
        return super().__new__(cls,*args,tzinfo=tzinfo)

    def add(self,years=0,months=0,**kw):
        '''增加日期，返回一个新的日期实例'''
        year,month=self.timetuple()[:2]
        year,month=divmod((year+years)*12+month+months-1,12)
        try:
            date=self.replace(year=year,month=month+1)
        except:
            # 加上年或月之后，如果没有对应的日，则以最后一天为准
            date=self.replace(year=year,month=month+2,day=1)-1
        return date+dt.timedelta(**kw)
        
    def __add__(self,days):
        '''增加日期，支持整数型'''
        if isinstance(days,int):
            days=dt.timedelta(days=days)
        return datetime(super().__add__(days))

    def __sub__(self,days):
        '''减少日期，支持整数类型'''
        if isinstance(days,(dt.datetime,dt.time)):
            return super().__sub__(days)
        return self.__add__(-days)

    @property
    def first_day_of_year(self):
        return datetime(self.year,1,1,tzinfo=LTZ)

    @property
    def last_day_of_year(self):
        return datetime(self.year,12,31,tzinfo=LTZ)

    @property
    def first_day_of_quartor(self):
        month=(self.quartor-1)*3+1
        return datetime(self.year,month,1,tzinfo=LTZ)

    @property
    def last_day_of_quartor(self):
        return self.first_day_of_quartor.add(months=3)-1
        
    @property
    def first_day_of_month(self):
        '''当月第一天'''
        return self.replace(day=1)

    @property
    def last_day_of_month(self):
        '''当月最后一天'''
        return self.add(months=1).first_day_of_month-1

    @property
    def is_weekend(self):
        '''是否为周末'''
        return self.weekday()>4

    @property
    def quartor(self):
        '''当前的季度，从1开始'''
        return (self.month-1) // 3 +1

    def format(self,fmt):
        '''格式化'''
        if '%Q' in fmt:
            fmt=fmt.replace('%Q','%s-%s'%(self.year,self.quartor))
        if '%q' in fmt:
            fmt=fmt.replace('%q',str(self.quartor))
        return super().strftime(fmt)

    # 使用date%'%Y-%m-%d'的语法来格式化日期
    __mod__= strftime = format
    
    def iter(self,end,step={'days':1},fmt=lambda x:x):
        '''遍历日期，如果days 为整数，则遍历days 指定的天数,
        若days 为非整数，则days 应为终止的日期,
        fmt 为返回格式：如为字符串，则格式化日期；若为可调用对象，则调用该日期'''
        if isinstance(fmt,str):
            _fmt=fmt
            fmt=lambda x:x.strftime(_fmt)
            
        if isinstance(end,dict):
            p,a={},{}
            for k,v in end.items():
                if k in set(['year','month','day']):
                    p[k]=v
                else:
                    a[k]=v
            end_day=self.add(**a) if a else self
            if a:
                end_day=end_day.replace(**p)
        else:
            end_day=datetime(end)
        while self<end_day:
            yield fmt(self)
            self=self.add(**step)

def date_add(dt,*args,**kw):
    '''日期及时间的加减,
    支持的参数有：years,months,days,hours,minutes,seconds'''
    return datetime(dt).add(*args,**kw)

def now(tz=LTZ):
    return datetime(dt.datetime.now(tz))



# 项目：标准库函数
# 模块：运行库
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2016-04-13 20:46

import os

class classproperty:
    '''类属性，用法：
    class A:
        @classproperty
        def name(cls):
              return cls.__name__

    A.name
    A().name
    '''
    def __init__(self,getter):
        self.getter=getter

    def __get__(self,instance,kclass):
        return self.getter(kclass)

class cachedproperty:
    '''类属性，用法：
    class A:
        @classproperty
        def name(cls):
              return cls.__name__

    A.name
    A().name
    '''
    def __init__(self,getter):
        self.getter=getter
        self.cache={}

    def __get__(self,instance,kclass):
        if kclass not in self.cache:
            self.cache[kclass]=self.getter(kclass)
        return self.cache[kclass]
    
def read_shell(cmd):
    '''
    执行系统命令，并将执行命令的结果通过管道读取。
    '''
    with os.popen(cmd)as fn:
        k=fn.read()
    return k.splitlines()

def write_shell(cmd,lines):
    '''
    执行系统命令，将指定的文通过管道向该程序输入。
    '''
    with os.popen(cmd,'w') as fn:
        if isinstance(lines,str):
            fn.write(lines)
        elif type(lines)in(tuple,list):
            [fn.write('%s\n'%(x))for x in lines]

def exec_shell(cmd):
    '''
    执行系统命令。
    '''
    return os.system(cmd)

def wlen(s):
    '''
    用于统计字符串的显示宽度，一个汉字或双字节的标点占两个位，
    单字节的字符占一个字节。
    '''
    return sum([2 if ord(x)>127 else 1 for x in s])
        
_des=None

def __get_des():
    from .pyDes import des,PAD_PKCS5
    global _des
    if _des is None:
        _des=des(key='huangtao',padmode=PAD_PKCS5)
    return _des

def encrypt(pwd):
    '''
    可逆加密程序。
    '''
    b=__get_des().encrypt(pwd)
    return "".join(['%02X'%(x)for x in b])

def decrypt(code):
    '''
    解密程序。
    '''
    b=__get_des().decrypt(bytes.fromhex(code))
    return b.decode('utf8')

def get_py(s,style=4,sep=''):
    '''
    获取拼音字母。
    '''
    from pypinyin.core import phrase_pinyin
    return sep.join([x[0] for x in phrase_pinyin(s,style,None)])

class _PY(type):
    def __truediv__(self,s):
        return get_py(s)

    def __or__(self,s):
        return get_py(s,style=0,sep=' ')
    
class PY(metaclass=_PY):
    '''以一种高逼格的方式获取拼音
    获取拼音首字母：  PY/'我们'   ===>   'wm'
    获取拼音：       PY|'我们'   ===>    'wo men'
    '''
    pass

def split(data,size=1000):
    '''拆分数据，其中datas应为list,size为每批数据的数量'''
    length=len(data)
    i=0
    for i in range(size,length,size):
        yield data[i-size:i]
    else:
        yield data[i:]

def deprecation(func,replace=''):
    '''DeprecationWarning'''
    import warnings
    message="%s 已被弃用"%(func)
    if replace:
        message+="，请使用 %s 替代"%(replace)
    warnings.warn(message,DeprecationWarning, stacklevel=2)



# 项目：标准库函数
# 模块：参数处理模块
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2016-03-16 15:27

'''
参数处理模块。
'''
from argparse import ArgumentParser
import sys
from .htutil import *

class Argument():
    def __init__(self,*args,**kwargs):
        self.args=args
        self.kwargs=kwargs
        
    def add(self,parser):
        parser.parser.add_argument(*self.args,**self.kwargs)

Arg=Argument

class Parser():
    def __init__(self,*items,proc=None,allow_empty=False,**kwargs):
        deprecation("Parser","arg 和 command")
        self.items=items
        self.proc=proc
        self.kwargs=kwargs
        self.allow_empty=allow_empty
        self._subparsers=None

    def __call__(self,argv=None):
        self.parser=ArgumentParser(**self.kwargs)
        for item in self.items:
            item.add(self)
        argv=argv or sys.argv[1:]
        if self.allow_empty or argv:
            ns=self.parser.parse_args(argv)
            if self.proc:
                if isinstance(self.proc,str):
                    self.proc=eval(self.proc)
                args=dict(ns._get_kwargs())
                if self._subparsers:
                    subcommand=args.pop('subcommand',None)
                    if subcommand and hasattr(self.proc,subcommand):
                        self.proc=getattr(self.proc,subcommand)
                self.proc(**args)
        else:
            self.parser.print_usage()

    @property
    def subparsers(self):
        if not self._subparsers:
            self._subparsers=self.parser.add_subparsers(
                dest="subcommand",help="sub commands")
        return self._subparsers

class SubParser(Parser):
    def __init__(self,name,*items,help=None,**kwargs):
        self.name=name
        self.help=help
        super().__init__(*items,**kwargs)
        
    def add(self,parser):
        self.parser=parser.subparsers.add_parser(
            self.name,help=self.help,**self.kwargs)
        for item in self.items:
            item.add(self)
            



# 项目：标准库函数
# 模块：markdown扩展
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2016-03-24 19:57

import mistune
import re
class Extension:
    gramar=r''
    name=''
 
    def output(self,*args,**kwargs):
        return ""
    
class InlineExtension(Extension):
    @classmethod
    def update(cls,markdown):
        setattr(markdown.inline.rules,
                cls.name,re.compile(cls.grammar))
        setattr(markdown.inline.__class__,
                'output_%s'%(cls.name),cls.output)
        markdown.inline.default_rules.append(cls.name)

class BlockExtension(Extension):
    def parse(self,*args,**kwargs):
        pass
    
    @classmethod
    def update(cls,markdown):
        setattr(markdown.block.rules,
            cls.name,re.compile(cls.grammar))
        setattr(markdown.block.__class__,
                'output_%s'%(cls.name),cls.output)
        markdown.block.default_rules.insert(0,cls.name)
        setattr(markdown.block.__class__,
                'parse_%s'%(cls.name),cls.parse)
                
class Markdown(mistune.Markdown):
    def __init__(self,extensions=None,**kw):
        super().__init__(**kw)
        if extensions:
            for extension in extensions:
                extension.update(self)
                
class Toc(BlockExtension):
    grammar=r'^\[toc\](\n+|$)'
    name='toc'
    def parse(self,m):
        self.tokens.append({'type':'toc'})

    def output(self):
        return ""

def get_links(text):
    parser=mistune.BlockLexer()
    parser(text)
    return parser.def_links

class BlockLexer(mistune.BlockLexer):
    def __call__(self,*args,**kw):
        result=super().__call__(*args,**kw)
        for k,v in self.def_links.items():
            v['link']='http://localhost/image/%s'%(v['link'])
        return result

s='''
This is a picture.

![sf]

[sf]: a/b.jpg "hello"
'''

m=Markdown(block=BlockLexer())
print(m(s))



# 项目：标准库函数
# 模块：协程模块
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2016-12-26 13:34

'''
对asyncio库进行进一步封装，主要功能如下：

1. 执行一个或并行运行多个协程：
    start(coro1,coro2,...)
2. 中断协程，并等待另一个协程的运行结果：
    result=await coro()
3. 在协程中，并发多个协程：
    await wait(*coros)
'''
import asyncio

__all__='start','wait','wait_for','sleep','iscoroutine','batch','run'

wait=asyncio.wait
wait_for=asyncio.wait_for
sleep=asyncio.sleep
iscoroutine=asyncio.iscoroutine

def batch(coro,*datas):
    # 使用一个协程函数批量生成协程
    return wait(map(coro,*datas))
    
def start(*coros):
    # 执行协程，如coro为多个，则全部并发执行。
    try:
        loop=asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        coro=coros[0] if len(coros)==1 else wait(coros)
        loop.run_until_complete(coro)
    finally:
        loop.close()

def run(*coros):
    loop=asyncio.get_event_loop()
    coro=coros[0] if len(coros)==1 else wait(coros)
    loop.run_until_complete(coro)



#!/usr/bin/env python3
# 项目：标准库
# 模块：版本管理程序
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2015-05-20 15:32
# 修订：2016-9-6 将其迁移至orange 库，并移除对stdlib 的依赖
# 修订：2017-2-10 pyver 增加 -y 功能，与远程服务器同步

import os
import sys
from orange import *
from orange.version import Ver
from orange.parseargs import *
from .pytools import pyupload,pysdist

class VersionMgr:
    branch=None   # 当前git分支
    up_to_date=True  # 与版本库是否同步
    untracted_files=[]  # 未跟踪文件 
    not_staged=[]       # 已更新文件
    to_be_commited=[]   # 待提交文件
    is_clean=False      # 工作区是否干净
    ver=None            # 当前程序版本
    repository=True     # 是否纳入git版本管理
    file_type=None
    def proc_git(self):
        from re import compile
        patterns={
            compile(r'On branch (?P<branch>\w+)'):
            lambda branch:setattr(self,'branch',branch),
            compile(r"Your branch is up-to-date with '.*?'."):
            lambda :setattr(self,'up_to_date',True),
            compile(r"Changes not staged for commit:"):
            lambda :setattr(self,'file_type','not_staged'),
            compile(r"Untracked files:"):
            lambda :setattr(self,'file_type','untracted_files'),
            compile(r"Changes to be committed:"):
            lambda :setattr(self,'file_type','to_be_commited'),
            compile(r"\t(.*?:\s*)?(?P<file>.*)"):
            lambda file:getattr(self,self.file_type).append(file),
            compile(r"nothing to commit"):
            lambda :setattr(self,'is_clean',True)
            }
            
        s=read_shell('git status')  # 读取git状态
        if s ==[]:
            self.repository=False   
            return
        for line in s:
            for k,v in patterns.items():
                r=k.match(line)
                if r:
                    v(**r.groupdict())
                    break

    def __init__(self):
        if Path('.git').is_dir():
            self.ver=Ver.read_file()
            if self.ver:
                self.proc_git()

    def write_version_file(self,ver):
        self.ver.write_file()
        
    def show_version(self):     # 显示版本号与git状态
        if self.repository:
            print('\n当前分支： %s'%(self.branch))
            print('远程版本最否最新：%s'%(self.up_to_date))
            print('工作区是否干净：%s'%(self.is_clean))
            if self.ver:
                print('当前版本文件名：%s'%(self.ver.filename))
                print('当前程序版本：%s'%(self.ver))
        else:
            print('没有纳入GIT管理')
    
    def commit_(self):        # 提交变更
        if self.branch=='master':
            raise Exception('错误：当前git分支必须不能为master')
        if self.is_clean:
            raise Exception('错误：当前工作区无待提交的更改')
        if not self.ver.prerelease: 
            raise Exception('错误：当前版本为最终版')
        if self.untracted_files:
            print('下面的文件没有被纳入git监控:')
            [print('\t%s'%(file_name)) for file_name in \
                   self.untracted_files]
            cmd=None
            while cmd not in ('a','A','y','Y','n','N'):
                cmd=input(
                    '请选择： Y-全部跟踪,N-全部不跟踪,A-放弃操作：')
                if cmd in ('a','A'):
                    print("放弃本次操作，程序退出")
                    return
                elif cmd in ('y','Y'):
                    self.not_staged.extend(self.untracted_files)
                    
        if self.not_staged:
            print('以下文件将被提交到git')
            [print('\t%s'%(fil)) for fil in self.not_staged]
            [exec_shell('git add "%s"'%(fil)) for fil in\
                        self.not_staged]
            self.to_be_commited.extend(self.not_staged)
            
        if self.to_be_commited:
            if self.ver:
                ver=str(self.ver)
                self.ver.upgrade()
                print('版本号由%s升级到%s'%(ver,self.ver))
                self.ver.write_file()
                exec_shell('git commit -a -m "%s"'%(self.commit))
                exec_shell('git push --all')
                    
    def upgrade_ver(self):
        from distutils.version import StrictVersion as Version
        ver=self.ver
        if self.branch=='master':
            raise Exception('错误：当前git分支必须不能为master')

        if self.upgrade not in ('major','minor','micro','dev','p','patch',
                                'm','n','o','d'):
            raise Exception('错误：参数输入错误')

        if not self.is_clean and self.upgrade in ('d','dev',):
                raise Exception('错误：当前工作区有待提交的更改')

        if self.upgrade in ('d','dev')and ver.prerelease is None:
            raise Exception('错误：已经是最终版本')

        if self.upgrade not in ('d','dev')and ver.prerelease:
            raise Exception('错误：当前版本非最终版')

        if self.ver:
            self.ver.upgrade(self.upgrade)
            print('版本号由%s升级到%s'%(ver,self.ver))
            self.ver.write_file()
            ver=self.ver
            if ver.prerelease is None:
                cmds=['git commit -a -m "升级到最终版"',
                      'git checkout master',
                      'git merge %s'%(self.branch),
                      'git tag ver%s'%(self.ver),
                      'git checkout %s'%(self.branch),
                      'git push --all',
                      'git push --tags',
                      'pysdist',
                      'pysetup']
                for cmd in cmds:
                    exec_shell(cmd)

    def sync(self):
        s=read_shell('git status')
        if 'working directory clean' not in s[-1]:
            print(*s,sep='\n')
        elif "Your branch is ahead of" in s[1]:
            os.system('git push --all')
        else:
            os.system('git pull --all')
        
    @classmethod
    @arg('-u','--upgrade',nargs='?',action='store',
                metavar='segment',help=('升级版本号，可以为major,'
                'minor,micro,dev'))
    @arg('-c','--commit',nargs='?',metavar='message',help='提交变更')
    @arg('-s','--show',action='store_true',help='查看当前版本状态')
    @arg('-y','--sync',action='store_true',help='同步程序')
    @arg('-d','--sdist',action='store_true',help='源代码打包')
    @arg('-U','--upload',action='store_true',help='打包上传')
    def main(cls,show=None,upgrade=None,commit=None,sync=False,
                 sdist=False,upload=False):
        obj=cls()
        if obj:
            if show:
                obj.show_version()
            if commit:
                obj.commit=commit
                obj.commit_()
            if upgrade:
                obj.upgrade=upgrade
                obj.upgrade_ver()
            if sync:
                obj.sync()
            if upload:
                pyupload()
            if sdist:
                pysdist()
                



# 项目：标准库函数
# 模块：Windows注册表读写模块
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2017-03-15 20:35

# 警告：
#     编写跨平台程序时，不要直接引用本模块，正确的写法如下：
#     if os.name=='nt':
#         import regkey
#       


from winreg import *

__all__='RegKey','HKLM','HKCU','HKU','REG_BINARY',\
        'REG_DWORD','REG_EXPAND_SZ','REG_SZ'

class RegKey(object):
    __slots__='_items','_key','_subkey'
    def __init__(self,key,subkey=None):
        self._items={}
        self._key=key
        self._subkey=subkey
        
    def open(self):
        if self._subkey:
            subkey=self._subkey.replace('/','\\')
            self._key=CreateKey(self._key,subkey)
            self._subkey=None
        return self

    __enter__=open
        
    def __truediv__(self,subkey):
        if self._subkey:
            # 未打开的返回自身
            self._subkey='%s\\%s'%(self._subkey,subkey)
            return self
        else:
            # 已调用open
            return RegKey(self._key,subkey)

    def close(self):
        if hasattr(self._key,'Close'):
            self._key.Close()
    
    def __exit__(self,*args):
        self.close()

    def __getitem__(self,name):
        if name not in self._items:
            try:
                val=QueryValueEx(self._key,name)
            except:
                val=None,None
            self._items[name]=val
        return self._items.get(name)

    @property
    def value(self):
        return QueryValue(self._key,None)

    @value.setter
    def value(self,val):
        return SetValue(self._key,REG_SZ,val)
        
    def __setitem__(self,name,value):
        if isinstance(value,(tuple,list))and len(value)==2:
            if SetValueEx(self._key,name,0,*reversed(value)):
                self._items[name]=value
        else:
            raise Exception('参数格式不正确，应为value,Type')

    def __delitem__(self,name):
        DeleteValue(self._key,name)

    def iter_keys(self,func=EnumKey):
        i=0
        try:
            while 1:
                yield func(self._key,i)
                i+=1
        except:
            pass

    def iter_values(self):
        return self.iter_keys(func=EnumValue)

HKLM=RegKey(HKEY_LOCAL_MACHINE)
HKCU=RegKey(HKEY_CURRENT_USER)
HKU=RegKey(HKEY_USERS)



# 项目：标准函数库
# 模块：Python相关实用命令
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2015-08-09 07:50
# 修订：2016-03-19
# 修订：2016-5-12
# 修订：2016-9-23 新增pyupload功能，使用run_setup来进行安装调用

import sys
import os
from orange import Path,exec_shell
from orange.deploy import *

def _clear():
    for path in Path('.').glob('*.egg-info'):
        print('Path %s has beed deleted!'%(path))
        if path.is_dir():
            path.rmtree()
            
def pyupload():
    import os
    cmd='setup' if os.name=='nt' else 'python3 setup.py'
    cmd='%s sdist --dist-dir %s upload'%(cmd,Path('~/OneDrive/pylib'))
    #run_setup('sdist','--dist-dir',str(Path('~/OneDrive/pylib')),'upload')
    os.system(cmd)
    _clear()

def pysdist():
    run_setup('sdist','--dist-dir',str(Path('~/OneDrive/pylib')))
    _clear()




# 项目：Python开发工具包
# 模块：工具包
# 作者：黄涛
# 创建：2015-9-2

from .path import *
from .version import *
from .deploy import *
from .debug import *
from .htutil import *
from .dateutil import *
from .regex import *
from .mail import *
from .click import *

__all__='get_ver','Path','get_path',\
  'first','last','Ver','decode',\
  'setup','decorator','trace','config_log','ensure','info',\
  'classproperty','is_installed','is_dev','cachedproperty',\
  'read_shell','write_shell','exec_shell','wlen',\
  'encrypt','decrypt','get_py','split','deprecation',\
  'LOCAL','UTC','now','datetime','fprint','date_add','ONEDAY','LTZ',\
  'ONESECOND','R','sendmail','tsendmail','Mail','PY','MailClient',\
  'convert_cls_name','verbose','arg','command'




# 项目：标准库函数
# 模块：程序版本模块
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2016-03-11 12:59
# 修订：2016-09-10 增加Ver 类来管理版本
from orange.path import Path
from distutils.version import StrictVersion
from orange.debug import *

SEGMENT={'major':0,'m':0,
        'minor':1,'n':1,
        'micro':2,'o':2,
        'patch':2,'p':2,
        'dev':3,'d':3,
        '#':4,}

first=lambda x:x and x[0]
last=lambda x:x and x[-1]

class Ver(StrictVersion):
    '''自定义版本管理程序
    增加upgrade功能
    '''
    def upgrade(self,segment=4):
        '''升级版本号
        其中参数 segment可以为：
        major:升级主版本号
        minor:升级小版本号
        patch:升级补丁版本号
        dev:升级开发版本号
        #:升级开发版本序号
        '''
        if isinstance(segment,str):
            segment=SEGMENT.get(segment.lower(),4)
        if segment<3:
            ensure(not self.prerelease,
                   '开发版本号不允许直接升级正式版本号！')
        else:
            ensure(self.prerelease,'正式版本号不允许升级开发版本号')
        if segment==4:
            self.prerelease=self.prerelease[0],self.prerelease[1]+1
        elif segment==3:
            if self.prerelease[0]=='a':
                self.prerelease='b',1
            else:
                self.prerelease=None
        else:
            self.prerelease='a',1
            nv=list(self.version)
            nv[segment]+=1
            for x in range(segment+1,3):
                nv[x]=0
            self.version=tuple(nv)
        return self

    @classmethod
    def read_file(cls,filename=None):
        '''从版本文件中读取版本号'''
        if not filename:
            filename=first(list(Path('.').glob('*/__version__.py')))
        if filename:
            for line in Path(filename).lines:
                if line.startswith('version'):
                    ver=cls(line.split('"')[1])
                    ver.filename=filename
                    return ver

    def write_file(self,filename=None):
        '''将版号写入版本文件中'''
        filename=filename or self.filename
        Path(filename).text='version="%s"\n'%(self)
                    
def get_ver():
    '''该函数将会被废弃'''
    ver_file=first(list(Path(".").glob("*/__version__.py")))
    if ver_file:
        for line in ver_file.lines:
            if line.startswith('version'):
                return line.split('"')[1]

def upgrade_ver(ver,segment='#'):
    '''该函数将被废弃'''
    if not isinstance(ver,Ver):
        ver=Ver(ver)
    if isinstance(segment,str):
        segment={'major':0,
                 'm':0,
                 'minor':1,
                 'n':1,
                 'micro':2,
                 'o':2,
                 'dev':3,
                 'd':3,
                 '#':4,}.get(segment.lower(),4)
    if(ver.prerelease and segment<3)or(not ver.prerelease and segment>=3):
        raise Exception('版本升级失败')
    if segment==4:
        ver.prerelease=ver.prerelease[0],ver.prerelease[1]+1
    elif segment==3:
        if ver.prerelease[0]=='a':
            ver.prerelease='b',1
        else:
            ver.prerelease=None
    else:
        ver.prerelease='a',1
        nv=[0,0,0]
        nv[:segment+1]=ver.version[:segment+1]
        nv[segment]+=1
        ver.version=tuple(nv)
    return ver
        
            



# 项目：标准库函数
# 模块：配置mongodb数据库
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2016-11-19 10:18

import sys
from orange.deploy import *
from orange import exec_shell,read_shell

SERVERNAME='MongoDb'
def win_deploy():
    root=get_path(SERVERNAME,False)[0]
    data=root / 'data'
    data.ensure()
    result=read_shell('sc query %s'%(SERVERNAME))
    if 'SERVICE_NAME:' in result[1]:    # 检查服务是否安装
        print('%s 服务已安装！'%(SERVERNAME))
        if 'STOPPED' in result[3]:
            exec_shell('sc start %s'%(SERVERNAME))
        return
    
    cmd=('mongod','--install',     # 作为服务安装
         '--serviceName %s'%(SERVERNAME),  # 服务名称
      '--logpath %s\\mongodb.log'%(root),   # 日志文件
      '--logappend',                        # 日志采用添加模式
      '--dbpath %s'%(data),                 # 数据文件目录
      '--directoryperdb',                   # 每个数据库采用单一文件
      )
    r=exec_shell(' '.join(cmd))
    exec_shell('sc start %s'%(SERVERNAME))
    if r==0:
        print('%s 服务安装成功！'%(SERVERNAME))

def darwin_deploy():
    pass
    
def main():
    import sys
    if sys.platform=='win32':
        win_deploy()
    elif sys.platform=='darwin':
        darwin_deploy()
    else:
        print('操作系统%下的配置未实现')

if __name__=='__main__':
    main()



# 项目：标准程序库
# 模块：发送电子邮件
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2016-10-26 10:25

from email.mime.text import MIMEText
from email.message import Message
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.header import Header
from orange import *
import smtplib
import io
import base64

def sendmail(*messages):
    '''发送邮件'''
    with MailClient() as client:
        for message in messages:
            message.post(client)

def tsendmail(*args):
    '''采用线程发送邮件'''
    from threading import Thread
    Thread(target=sendmail,args=args).start()

class MailClient(smtplib.SMTP):
    '''构造邮件客户端，使用方法如下：
       client=MailClient(host,user,passwd)
    '''
    config={}   # 用于配置发送邮件的想着参数，如：host,user,passwd
    def __init__(self,host=None,user=None,passwd=None,*args,**kw):
        host=host or self.config.get('host')
        user=user or self.config.get('user')
        passwd=passwd or self.config.get('passwd')
        super().__init__(host,*args,**kw)
        self.login(user,passwd)

    def Mail(self,*args,**kw):
        m=Mail(*args,client=self,**kw)
        if m.sender is None:
            m.sender=self.config.get('sender')
        return m

class Mail:
    '''创建电子邮件，使用方法如下：
    mail=Mail(sender,to,subject,body,cc,bcc)
    添加图片：
    mail.add_image(filename,cid)
    添加文件附件：
    mail.add_file(filename)
    通过流附加文件：
    mail.add_fp(fp,filename)
    发送邮件：
    mail.post(client)
    '''
    def __init__(self,sender=None,to=None,subject=None,body=None,
                 cc=None,bcc=None,client=None):
        '''初绍化邮件'''
        self.attachments=[]
        self.subject=subject
        self.to=to
        self.sender=sender
        self.body=body
        self.cc=cc
        self.bcc=bcc
        self.client=client

    @property
    def message(self):
        '''获取邮件的MESSAGE属性'''
        body=MIMEText(self.body,'html','utf-8')
        if self.attachments:
            msg=MIMEMultipart()
            msg.attach(body)
            for attachment in self.attachments:
                msg.attach(attachment)
        else:
            msg=body
        msg['Subject']=self.subject
        msg['To']=self.to
        msg['Sender']=self.sender
        if self.cc:
            msg['Cc']=self.cc
        if self.bcc:
            msg['Bcc']=self.bcc
        return msg
        
    def __str__(self):
        return self.message.as_string()

    def add_file(self,filename):
        fn=Path(filename)
        ensure(fn.is_file(),'文件不存在！')
        self.add_fp(fn.open('rb'),fn.name)

    def add_fp(self,fp,filename,encoding='utf8'):
        filename='=?utf-8?b?%s?='%(base64.b64encode(
            filename.encode('UTF-8')).decode('utf-8'))
        if callable(fp):
            with io.BytesIO() as _fp:
                fp(_fp)
            fp=_fp
        fp.seek(0)
        a=MIMEText(fp.read(),'base64',encoding)
        a['Content-Type']='application/octet-stream'
        a['Content-Disposition']='attachment; filename= %s'%(filename)
        self.attachments.append(a)

    def add_image(self,filename,cid=None):
        with open(filename,'rb')as fn:
            msg=MIMEImage(fn.read())
            if cid:
                kw['Content-ID']=cid
            self.attachments.append(msg)

    def post(self,mailclient=None):
        mailclient=mailclient or self.client
        if mailclient:
            mailclient.send_message(self.message)

if __name__=='__main__':
    body='''
<h2>This is a test</h2>
我们是一群来自北方的儿儿狼。
'''
    msg=Mail('huangtao@czbank.com','huangtao@czbank.com','这是一封邮件',body)
    msg.add_file('~/abc.xlsx')

    from io import *
    with StringIO() as txt:
        txt.write('我们要去哪里？\n')
        txt.write('他们要干啥?\n')
        msg.add_fp(txt,'abc.txt')
    from orange.xlsx import *
    with BytesIO() as fn:
        with Book(fn) as book:
            book.worksheet='Sheet1'
            book.A1_D1='This is a title','title'
            book.A2=['姓名','年龄'],'h2'
        msg.add_fp(fn,'vacation.xlsx') 
    tsendmail('zhmail.czbank.com','huangtao','zxcv9169',msg)
    print('ok')
    
    



# 项目：标准库函数模块
# 模块：配置程序
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2015-05-26 16:12
# 修订：2016-10-28 增加

import os
import sys
import sysconfig
import orange
import atexit
from configparser import ConfigParser
from orange import *

class Config:
    def __init__(self,config_path=None,data_path=None,
                 is_dev=None,project=None):
        if is_dev is None:
            self.is_dev=orange.is_dev()
        else:
            self.is_dev=is_dev
        if project is None:
            self.project=os.path.splitext(
                os.path.basename(sys.argv[0]))[0]
        else:
            self.project=project
        self.os_name=os.name
        if self.os_name=='posix':
            config_ext='.conf'
            self.config_path=os.path.expanduser(
                "~/.%s"%(self.project))
        else:
            config_ext='.ini'
            self.config_path=os.path.join(os.getenv("APPDATA"),
                                          self.project)
        if self.is_dev:
            self.config_path=os.path.abspath("appdata")
        else:
            if config_path:
                self.config_path=config_path
        if not hasattr(self,"data_path"):
            self.data_path=self.config_path
        self.config_file=os.path.join(self.data_path,
                            self.project+config_ext)
        self.modified=False
        self.load_config()
        
    def load_config(self,files=None):
        if files is None:
            files=self.config_file
        if not hasattr(self,'parser'):
            self.parser=ConfigParser()
        self.parser.clear()
        self.parser.read(files,encoding='utf8')
        self.modified=False
        if isinstance(files,str):
            self.cur_file=files
        else:
            self.cur_file=files[-1]

    def __setitem__(self,name,data):
        return self.update(name,data)

    @property
    def sections(self):
        return self.parser.sections()
    
    def get(self,section):
        if self.parser.has_section(section):
            d={}
            for option,value in self.parser.items(section):
                if self.is_passwd(option):
                    value=decrypt(value)
                d[option]=value
            return d

    def update(self,section,data):
        self.modified=True
        if not self.parser.has_section(section):
            self.parser.add_section(section)
        for option,value in data.items():
            if self.is_passwd(option):
                value=encrypt(value)
            self.parser.set(section,option,str(value))
        
    def get_many(self,*sections):
        return dict([(section,self.get(section)) for section in sections])
    
    def update_many(self,datas):
        for section,data in datas.items():
            self.update(section,data)
            
    def is_passwd(self,key):
        return key.lower() in ('passwd','password')        

    def save_config(self):
        # ensure_path(os.path.dirname(self.cur_file))
        Path(self.cur_file).parent.ensure()
        if self.modified:
            with open(self.cur_file,'w',encoding='utf8')as fn:
                self.parser.write(fn)

    def init_logging(self):
        import logging
        file_name=os.path.join(self.data_path,
                               self.project+'.log')
        if self.is_dev:
            level='DEBUG'
        else:
            level='WARN'
        default={
            'filename':file_name,
            'level':level,
            'format':'%(asctime)s %(levelname)s\t%(message)s',
            'datefmt':'%Y-%m-%d %H:%M'}
        default.update(self.get('logging'))
        logging.basicConfig(**default)

_config=None

def config(*args,**kw):
    global _config
    if _config is None:
        _config=Config(*args,**kw)
    return _config

@atexit.register
def save_config():
    if _config and _config.modified:
        _config.save_config()

if __name__=='__main__':
    config=config()
    print(config.config_file)
    config.load_config()
    d={
        'hello':{'HOST':'123456',
                 'USER':'fsda',
                 'PASSWD':'123456',
                 'DB':'hunter',
                 }
                 }
    config.update_many(d)
    f=config.get_many('hello')
    print(config.get('hello'))
    



# 项目：标准库函数
# 模块：公共参数查询
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2016-05-20 16:45

from orange.parseargs import *
from orange import *
import re

def extract_str(s):
    if s.startswith('"')and s.endswith('"'):
        s=s[1:-1]
    return s.strip()

def query_canshu(category,query):
    TYPE={'jym':'transactions_output.csv',
          'km':'ggkmzd.del',
          'jg':'ggjgm.del',
          'gy':'users_output.csv',
          'user':'users_output.csv',}
    root=Path('~/OneDrive/工作/参数备份/')
    files=[filename for filename in root.rglob(TYPE[category])]
    if files:
        filename=max(files)
        print('file "%s" was selected!'%(filename.name))
        for line in filename.lines:
            if query in line:
                print(",".join([extract_str(x)for x in\
                                line.split(',')]))

canshu=Parser(
    Argument('-c','--category'),
    Argument('query'),
    proc=query_canshu)



# 项目：Python开发工具包
# 模块：工具包
# 作者：黄涛
# 创建：2015-9-2
# 修订：2015-9-3 添加setup.py以及testing的相关初始化
# 修订：2016-9-6 移除stdlib的支持

import os
from orange import *
from orange.parseargs import *

INITIAL_VERSION='version="0.1"\n'
INITIAL_FILE='''# 项目：{project}
# 作者：{author}
# 邮件：{email}
# 创建：{date:%Y-%m-%d}
'''
SETUP_FILE='''#!/usr/bin/env python3
from orange import setup
requires=[
          ]
console_scripts=[
        # 'cmdname=package:function',
                 ]
gui_scripts=[
        # 'cmdname=package:function',
                 ]
setup(
        name='{prj_name}',
        author='{author}',
        author_email='{email}',
        platforms='any',
        description='{project}',
        install_requires=requires,
        long_description='{project}',
        entry_points={{'console_scripts':console_scripts,
                       'gui_scripts':gui_scripts}},
        url='https://github.com/huangtao-sh/{prj_name}.git',
        license='GPL',
        )
'''
TESTING_PKG='''from .test_sample import *
'''
TEST_SAMPLE='''import unittest
class TestSample(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_sample(self):
        a='this is a test.'
        self.assertEqual(a,a)
'''

def py_init(project='',author='',email=''):
    '''
    初始化项目文件
    自动在当前目录下生成包文件、初始版本号文件、安装文件和测试文件
    project:项目名称
    author:作者姓名
    email:作者的电子邮件
    '''
    prj_name=os.path.split(os.path.abspath('.'))[-1]
    ver_file='%s/__version__.py'%(prj_name)
    pkg_file='%s/__init__.py'%(prj_name)
    # 生成setup.py文件
    if not os.path.isfile('setup.py'):
        Path('setup.py').text=SETUP_FILE.format(\
            author=author,prj_name=prj_name,
            project=project,email=email)
        print('已生成setup.py')
        
    # 创建包目录
    if not os.path.isdir(prj_name):
        os.mkdir(prj_name)
    # 生成初始化版本文件    
    if not os.path.isfile(ver_file):
        Path(ver_file).text=INITIAL_VERSION
        print('当前程序版本初始化为：0.1')
    # 生成初始化包文件
    if not os.path.isfile(pkg_file):
        contents=INITIAL_FILE.format(project=project,
                                      author=author,
                                      email=email,
                                      date=now())
        Path(pkg_file).text=contents
        print('已初始化包文件')

    # 生成测试模板文件
    if not os.path.isdir('testing'):
        testing=Path('testing')
        testing.ensure()
        (testing / '__init__.py').text=TESTING_PKG
        (testing / 'test_sample.py').text=TEST_SAMPLE
        print('已创建测试模板文件')

main=Parser(Arg('-p','--project',help='项目描述'),
            Arg('-a','--author',help='作者姓名',default='黄涛'),
            Arg('-e','--email',help='电子邮件',
                default='hunto@163.com'),
            proc=py_init)



#!/usr/bin/env python3
# 项目：克隆github.com上的项目
# 模块：命令行模块
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2015-06-11 12:28
# 修订：2016-11-18 采用Parser来分析参数

#from stdlib import parse_args,exec_shell
from orange.parseargs import *
from orange import *
import sys

def proc(repos=None,user=None,protocol='SSH'):
    if user is None:
        from configparser import ConfigParser
        import os
        config=ConfigParser()
        config.read([os.path.expanduser('~/.gitconfig')])
        try:
            user=config.get('user','name')
        except:
            raise Exception('用户不存在！')
    protocol=protocol.upper()
    URL='git@github.com:%s'%(user) if protocol=='SSH' else \
      'https://github.com/%s'%(user)
    for repo in repos:
        url='%s/%s.git'%(URL,repo)
        print('cloning', url)
        exec_shell('git clone %s'%(url))   

main=Parser(
    Arg('repos',nargs='+',metavar='repo',help='要下载的软件仓库，可以为多个'),
    Arg('-p','--protocol',nargs='?',default='SSH',
        help='传输协议，可以为SSH或HTTPS'),
    Arg('-u','--user',nargs='?',help='要下载的用户名,默认为本人的仓库'),
    proc=proc)

if __name__=='__main__':
    main()

            



#!/usr/bin/env python3
from orange import setup

console_scripts=['conv=orange.path:convert',
                 #'pytest=orange.pytools:pytest',
                 'pysdist=orange.pytools:pysdist',
                 'pyupload=orange.pytools:pyupload',
                 'canshu=orange.ggcs:canshu',
                 #'mongodb=orange.mongodb:main',
                 'pyver=orange.pyver:VersionMgr.main',
                 'plist=orange.plist:main',
                 'pyinit=orange.init:main',
                 'gclone=orange.gclone:main',
                 'deploymongo=orange.mongodb:main',
                 'fkgfw=orange.fkgfw:main',
                 'sysinit=orange.sysinit:main']
setup(
        name='orange-kit',
        platforms='any',
        description='orange',
        long_description='orange',
        url='https://github.com/huangtao-sh/orange.git',
        cscripts=console_scripts,
        # entry_points={'console_scripts':console_scripts},
        license='GPL',
        )



# 项目：标准函数库
# 模块：业务测试模块
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2017-01-04 21:34

# Remember:
# Do not modify this file!!!
# 

import sys
sys.path.insert(0,'.')
from testing import *
unittest.main()




#!/usr/bin/env python3
# 项目：自用函数库
# 模块：校验位计算模块
# 作者：黄涛

def checksum(s,calc,Key,Sum):
    return Sum[sum([calc(c,k) for c,k in zip(s,Key)])%len(Sum)]

def circle(s):
    while True:
        for i in s:
            yield i
    
def id_card(card_no):
    '''修正居民身份证的校验位'''
    Key=(7,9,10,5,8,4,2,1,6,3,7,9,10,5,8,4,2)
    Sum='10X98765432'
    length=len(card_no)
    if 15==length:
        card_no='19'.join([card_no[:6],card_no[6:]])
        length+=2
    if 18>=length>=17:
        return card_no[:17]+checksum(card_no[:17],\
                        lambda x,y:int(x)*y,Key,Sum)

def org_code(code_no):
    '''修正组织机构代码证校验位'''
    key=(3,7,9,10,5,8,4,2)
    Sum='0X987654321'
    code_no=code_no[:8].upper()
    def calc(c,k):
        if '0'<=c<='9':
            return int(c)*k
        elif 'A'<=c<='Z':
            return (ord(c)-0X37)*k
        else:
            return 0
    return '-'.join([code_no,checksum(code_no,calc,key,Sum)])
    
def bank_card(card_no):
    '''修正银行卡校验位'''
    Sum='0987654321'
    if len(card_no) in (16,17,19):
        card_no=card_no[:-1]
        return card_no+checksum(reversed(card_no),
                        lambda x,y:sum(divmod(int(x)*y,10)),
                        circle([2,1]),Sum)

def czbank_ac(acno):
    '''账号校验位算法'''
    keys=30,3,50,5,70,7,10,11,30,13,70,17,90,19,30,23,90,29,10,31
    if len(acno)>=20:
        checksum='%02d'%(sum([int(x)*k for x,k in zip(acno,keys)]))
        if acno[11]=='9':
            return '%s%s'%(acno[:21],checksum[-2])
        else:
            return '%s%s'%(acno[:20],checksum[-2:])

def credit_code(code):
    Base="0123456789ABCDEFGHJKLMNPQRTUWXY"
    wi=1, 3, 9, 27, 19, 26, 16, 17, 20, 29, 25, 13, 8, 24, 10, 30, 28
    if len(code)>=17:
        check=sum([Base.index(k)*w for k,w in zip(code,wi)])%31
        if check:
            check=31-check
        return '%s%s'%(code[:17],Base[check])
        
    
if __name__=='__main__':
    print(id_card('310115201411142227'))
    print(id_card('412902197909022053'))
    print(bank_card('3568680095415995'))
    print(bank_card('6223093310810005226'))
    print(org_code('L2345678-1'))
    print(org_code('74371976-1'))
    print(credit_code('91460600MA5RC30P9Y'))
        



# 项目：标准库函数
# 模块：网络爬虫
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2016-12-26 17:08
# 修订：2017-01-14

from aiohttp import *
from orange.coroutine import *
from orange import *
from bs4 import BeautifulSoup as BS4

__all__='Crawler','wait','BS4'

class Crawler(ClientSession):
    root=''   # 网址的根目录
    def __init__(self,root=None,*args,**kw):
        if root:
            if root.endswith('/'):
                root=root[:-1]
            self.root=root    # 设置根目录
        super().__init__(*args,**kw)

    def _request(self,method,url,*args,**kw):
        # 处理网址，支持网址中只输入后续路径
        if ':' not in url:
            if url.startswith('/'):
                url=url[1:]
            url='/'.join([self.root,url])
        return super()._request(method,url,*args,**kw)
    
    async def get_text(self,url,*args,method='GET',**kw):
        # 以获取指定网页的文本
        async with self.request(method,url,*args,**kw)as resp:
            return await resp.text()

    async def get_soup(self,url,*args,**kw):
        # 获取指定网页的BeautifulSoup实例
        text=await self.get_text(url,*args,**kw)
        return BS4(text,'lxml')

    async def download(self,url,params=None,path='.',*args,**kw):
        # 下载文件，其中path可以是下载文件的文件名，也可以是下载目录
        async with self.get(url,params=params,*args,**kw)as resp:
            path=Path(path)
            if path.is_dir():
                from urllib.parse import unquote
                # 获取文件名字段
                filename=resp.headers['Content-Disposition'] 
                filename=filename.split(';')[-1]  # 获取最后一个参数
                tp,filename=filename.split('=')
                if tp.strip()=='filename*':
                    filename=(filename.split("'"))[-1]
                if filename.startswith('"'):
                    filename=filename[1:-1]
                filename=unquote(filename)
                path=path / Path(filename).name
            path.write(data=await resp.read()) # 写入文件
                    
    async def get_json(self,url,*args,encoding=None,method='GET',
                           **kw):
        # 获取指定网页的JSON数据
        async with self.request(method,url,*args,**kw)as resp:
            return await resp.json(encoding=encoding)

    async def run(self):
        # 爬虫的主运行程序
        raise Exception('This function doesn''t exist!')

    @classmethod
    def start(cls,target=None,args=None,kwargs=None,**kw):
        # target：待执行的命令，必须为协程，若无，则调用run
        # target的第一个参数应为 session
        # args：target的位置参数
        # kwargs: target的关键字参数
        # kw: cls的关键字参数
        args=args or []
        kwargs=kwargs or {}
        target=target or cls.run
        async def _main():
            async with cls(**kw)as sess:
                await target(sess,*args,**kwargs)
        start(_main())
 



# 项目：标准工具库
# 模块：正则表达式
# 作者：黄涛
# License:GPL
# Email:huangtao.sh@icloud.com
# 创建：2016-09-30 17:18

import re

_FLAGS='AILUMSX'

__all__='R','convert_cls_name'

class _R(type):
    '''Regex类的元类，在使用R/pattern时被调用，调用生成R类。
    其中pattern可以是str，也可以是list或tuple。
    如pattern为list或tuple，则其格式为 pattern,flag。
    flag的格式可以是"AILUMSX"或是整数。'''
    
    def __truediv__(self,pattern):
        if not isinstance(pattern,(list,tuple)):
            pattern=(pattern,)
        return R(*pattern)

class R(metaclass=_R):
    '''Regex类，主要有以下功能。
    1、生成
      r=R/pattern 或者 r=R/(pattern,flag)
    2、拆分
      for substr in R/pattern|str:
          print(substr)
    3、查找所有匹配记录
      for match in R/pattern//str:
          print(match.group())
    4、对Regex类进行操作
       可以使用match,search,findall,finditer函数进行操作。
       if (R/pattern).match(str):
          pass
    '''
    def __init__(self,pattern,flag=0):
        '''初始化，生成模板。'''
        if flag and isinstance(flag,str):
            n=0
            for i in flag.upper():
                if i in _FLAGS:
                    n|=getattr(re,i)
            flag=n
        self._regex=re.compile(pattern,flag)

    def __eq__(self,s):
        '''是否完全匹配。'''
        return self._regex.fullmatch(s)

    def __mod__(self,s):
        '''搜索匹配的记录'''
        return self._regex.search(s)
    
    def __truediv__(self,s):
        '''返回正则表达式的操作对象'''
        return RegOperation(self._regex,s)

    def __or__(self,s):
        '''对指定字符串进行拆分。'''
        return self._regex.split(s)

    def __floordiv__(self,s):
        '''查找所有匹配的记录。'''
        yield from self._regex.finditer(s)

    def __getattr__(self,name):
        '''对Regex对象进行操作，如match,search,findall,finditer等。'''
        return getattr(self._regex,name)

class RegOperation:
    '''正则表达式操作对象，具有以下功能：
    1、判断是否匹配。
      if R/pattern/str:
         pass
    2、查询所有匹配记录。
      for substr in R/pattern/str:
         print(substr)
    3、批量替换
      s=R/pattern/str%repl
    '''
    def __init__(self,regex,search):
        '''对象初始化，由R对象自动调用。'''
        self._regex=regex
        self._search=search

    def __bool__(self):
        '''判断是否匹配。'''
        return bool(self._regex.search(self._search))

    def __iter__(self):
        '''查找所有匹配项。'''
        yield from self._regex.findall(self._search)

    def __mod__(self,repl):
        '''字符串替换。'''
        if isinstance(repl,(list,tuple)):
            repl,count=repl
        else:
            count=0
        return self._regex.sub(repl,self._search,count)

# 将类名由 TestCase 格式转换为 test_case
convert_cls_name = lambda name:'_'.join([x.lower() for x in \
            R/'[A-Z][a-z0-9]*'/name])

            
